## SISCER 2022: Joint Modeling of Longitudinal and Survival Data
## Benjamin French, PhD; b.french@vumc.org

install.packages(c('JM', 'ggplot2', 'survminer', 'dplyr'), 
                 repos='https://cloud.r-project.org')

library(JM)
library(ggplot2)
library(survminer)
library(dplyr)

theme_set(theme_bw())
mytheme <- function(...){
  theme(panel.grid.minor=element_blank(),
        plot.margin=unit(c(0.5, 0.5, 0.5, 0.5), 'lines'),
        axis.title.x=element_text(size=12, vjust=0),
        axis.title.y=element_text(size=12, vjust=1, angle=90),
        legend.title=element_text(size=12, face='bold'),
        legend.text=element_text(size=12),
        legend.position='top', legend.direction='horizontal', 
        legend.key=element_blank(),
        legend.key.width=unit(1.5,"cm"),
        legend.key.height=unit(0.35,'cm'),
        strip.text.y=element_text(size=12))}

?pbc2
data(pbc2)
head(pbc2, 11)
str(pbc2)

subset(pbc2, !is.na(serBilir)) %>% 
  count(id, name='m') -> num
with(num, table(m))

pbc2 <- within(pbc2, {
  levels(drug) <- c('Placebo', 'Treatment')})

ggplot(subset(pbc2, year==0), aes(x=serBilir)) +
  geom_histogram(color='black', fill='white') +
  facet_wrap(~ drug) +
  scale_x_continuous(name='Serum bilirubin, mg/dl') +
  scale_y_continuous(name='Count') +
  mytheme()

ggplot(subset(pbc2, is.element(id, 1:16)), aes(year, serBilir)) +
  facet_wrap( ~ id) +
  geom_vline(aes(xintercept=years, color=factor(status!='dead')), lwd=1) + 
  geom_point() +
  geom_smooth(se=FALSE, method='lm') + 
  scale_x_continuous(name='Time, years', limits=c(0,15)) +
  scale_y_continuous(trans='log10', name='Serum bilirubin, mg/dl') + 
  scale_colour_manual(name='Status', breaks=c(FALSE, TRUE), values=c('red', 'darkgray'),
                      labels=c('Death', 'Censored')) +
  mytheme()

?geom_smooth

ggplot(pbc2, aes(year, serBilir, group=id)) +
  geom_line(aes(color=drug), alpha=0.4) +
  geom_smooth(aes(year, serBilir, group=drug, color=drug), se=FALSE, size=1.5) +
  scale_x_continuous(name='Time, years', limits=c(0,15)) +
  scale_y_continuous(trans='log10', name='Serum bilirubin, mg/dl') + 
  scale_color_manual(name='Intervention', values=c('#00BFC4', '#F8766D')) +
  mytheme()

# Main effects of treatment and time, random intercepts
ml1 <- lme(log(serBilir) ~ drug + year, random = ~ 1 | id, data=pbc2)
summary(ml1)

# Interaction between treatment and time, random intercepts
ml2 <- lme(log(serBilir) ~ drug*year, random = ~ 1 | id, data=pbc2)
summary(ml2)

# Treatment-specific time effects, random intercepts
ml3 <- lme(log(serBilir) ~ year + drug:year, random = ~ 1 | id, data=pbc2)
summary(ml3)

# Treatment-specific time effects, random intercepts + slopes
ml4 <- lme(log(serBilir) ~ year + drug:year, random = ~ year | id, data=pbc2)
summary(ml4)

# 95% confidence intervals
intervals(ml4)
exp(intervals(ml4)$fixed)

# Likelihood ratio test for inclusion of random slopes
anova(ml4, ml3)

data(pbc2.id)
head(pbc2.id, 2)

km <- survfit(Surv(years, status=='dead') ~ sex, data=pbc2.id)

ggsurvplot(km, data=pbc2.id, xlab='Time, years',
           conf.int=TRUE, risk.table=TRUE, risk.table.y.text=FALSE,
           ggtheme=theme(panel.grid.minor=element_blank()), tables.height=0.15,
           tables.theme=theme(panel.border = element_blank(), 
                              panel.grid.major=element_blank(),
                              axis.title.x=element_blank(),
                              axis.title.y=element_blank(),
                              axis.text.x=element_blank(),
                              axis.ticks.x=element_blank()),
           legend.title='Sex', legend.labs=c('Male', 'Female'),
           palette=c('#F8766D', '#00BFC4'))

# Standard Cox model with treatment; note x=TRUE argument
ms1 <- coxph(Surv(years, status=='dead') ~ drug + sex + I(age-50), data=pbc2.id, x=TRUE)
summary(ms1)

# Standard Cox model with treatment and baseline bilirubin
ms2 <- coxph(Surv(years, status=='dead') ~ drug + sex + I(age-50) + log(serBilir), data=pbc2.id)
summary(ms2)

# Check proportional hazards assumption
cox.zph(ms2)
plot(cox.zph(ms2)[4]) # Selects the fourth variable (bilirubin)

?tmerge
temp <- subset(pbc2.id, select=c(id, years, status, drug, sex, age, serBilir))
temp <- tmerge(temp, temp, id=id, event=event(years, status))
temp <- tmerge(temp, pbc2, id=id, serBilir=tdc(year, serBilir))
head(temp, 11)

pbc2 <- within(pbc2, {
  status.lab <- ifelse(status=='dead', 'Died', 'Censored')})

ggplot(pbc2, aes(year-years, serBilir, group=id)) +
  facet_wrap( ~ status.lab) +
  geom_line(aes(color=drug), alpha=0.4) +
  geom_smooth(aes(year-years, serBilir, group=drug, color=drug), se=FALSE, size=1.5) +
  scale_x_continuous(name='Time from end of follow-up, years', limits=c(-15,0)) +
  scale_y_continuous(trans='log10', name='Serum bilirubin, mg/dl') + 
  scale_color_manual(name='Intervention', values=c('#00BFC4', '#F8766D')) +
  mytheme()

# Extended Cox model with treatment and time-dependent serum bilirubin
ms3 <- coxph(Surv(tstart, tstop, event=='dead') ~ drug + sex + I(age-50) + log(serBilir), data=temp)
summary(ms3)

?jointModel
?jointModelObject

# Weibull hazard function
mj1 <- jointModel(lmeObject=ml4, survObject=ms1, timeVar='year', method='weibull-PH-aGH')
summary(mj1)

# Piecewise constant step function
mj2 <- jointModel(lmeObject=ml4, survObject=ms1, timeVar='year', method='piecewise-PH-aGH')
summary(mj2)

# Regression splines
mj3 <- jointModel(lmeObject=ml4, survObject=ms1, timeVar='year', method='spline-PH-aGH')
summary(mj3)

ml0 <- lme(log(serBilir) ~ year, random = ~ year | id, data=pbc2)
ms0 <- coxph(Surv(years, status=='dead') ~ sex + age, data=pbc2.id, x=TRUE)
mj0 <- jointModel(lmeObject=ml0, survObject=ms0, timeVar='year', method='weibull-PH-aGH')
anova(mj0, mj1)

?residuals.jointModel
?fitted.jointModel

plotResid <- function(x, y, col.lowess='red', ...){
  plot(x, y, ...)
  abline(h=0, col='gray', lwd=2)
  lines(lowess(x, y), col=col.lowess, lwd=2)}
plotQQ <- function(x, ...){
  qqnorm(x, main='')
  qqline(x, col='gray', lwd=2)}

r_l_s <- residuals(mj1, process='Longitudinal', type='stand-Subject')
f_l_s <- fitted(mj1, process='Longitudinal', type='Subject')
r_l_m <- residuals(mj1, process='Longitudinal', type='stand-Marginal')
f_l_m <- fitted(mj1, process='Longitudinal', type='Marginal')
r_s <- residuals(mj1, process='Event', type='Martingale')
f_s <- fitted(mj1, process='Longitudinal', type='EventTime')

par(mfrow=c(2,2), mar=c(5,5,1,1))
plotResid(f_l_s, r_l_s, xlab='Fitted values (subject)', ylab='Residuals (subject)')
plotQQ(r_l_s)
plotResid(f_l_m, r_l_m, xlab='Fitted values (marginal)', ylab='Residuals (marginal)')
plotResid(f_s, r_s, xlab='Fitted values (longitudinal)', ylab='Residuals (Martingale)')

# Multiple imputation residuals
set.seed(4)
M <- 5
diff.time <- with(pbc2, tapply(year, id, diff))
prev.y <- with(pbc2, tapply(log(serBilir), id, head, -1))
one.visit <- sapply(diff.time, length)==0
diff.time[one.visit] <- prev.y[one.visit] <- NA
dataVT <- data.frame(
  'id'=rep(names(prev.y), sapply(prev.y, length)),
  'diff.Times'=unlist(diff.time),
  'prev.y'=unlist(prev.y),
  'event'=1)
WeibFrl <- weibull.frailty(Surv(diff.Times, event) ~ prev.y, id='id', data=dataVT)
attr(WeibFrl, 'prev.y') <- 'prev.y'
resMargY <- residuals(mj1, process='Longitudinal', type='stand-Marginal')
fitMargY <- fitted(mj1, process='Longitudinal', type='Marginal')
resMI <- residuals(mj1, type='stand-Marginal', MI=TRUE, M=10, time.points=WeibFrl)
fitMargYmiss <- unlist(resMI$fitted.valsM)
resMargYmiss <- unlist(resMI$resid.valsM)
resMargYmi <- c(resMargY, resMargYmiss)
fitMargYmi <- c(fitMargY, fitMargYmiss)
datRes <- data.frame(resid=resMargYmi, fitted=fitMargYmi,
                     weight=c(rep(1, length(resMargY)), rep(1/M, length(resMargYmiss))))
fitLoess <- loess(resid ~ fitted, weights=weight, data=datRes)
nd <- data.frame(fitted=seq(min(datRes$fitted, na.rm=TRUE), 
                            max(datRes$fitted, na.rm=TRUE), 
                            length.out=100))
prdLoess <- predict(fitLoess, nd)

par(mfrow=c(1,1), mar=c(5,5,1,1))
plot(range(fitMargYmiss, na.rm=TRUE), range(resMargYmiss, na.rm=TRUE), 
     type='n', xlab='Fitted values (marginal)', ylab='Residuals (marginal)',
     ylim=c(-10,10))
points(fitMargYmiss, resMargYmiss, cex=0.5, col='gray')
points(fitMargY, resMargY)
abline(h=0, col='black', lwd=2)
lines(lowess(fitMargY, resMargY), col='red', lwd=2)
lines(nd$fitted, prdLoess, col='blue', lwd=2)

mj4 <- jointModel(lmeObject=ml4, survObject=ms1, timeVar='year', method='weibull-PH-aGH',
                  interFact=list(value = ~ drug, data=pbc2.id))
summary(mj4)

# Level and slope models for serum bilirubin in survival submodel
# Linear time effects
dFormL <- list(fixed = ~ 1 + drug, indFixed=c(2,3), random = ~ 1, indRandom=2)
mj5 <- jointModel(lmeObject=ml4, survObject=ms1, timeVar='year', method='weibull-PH-aGH',
                  parameterization='both', derivForm=dFormL)
summary(mj5)

# Quadratic time effects
ml6 <- lme(log(serBilir) ~ drug*(year + I(year^2)), random = ~ year + I(year^2) | id, data=pbc2)
dFormQ <- list(fixed = ~ 1 + I(2*year) + drug + I(2*year):drug, indFixed=3:6, 
               random = ~ 1 + I(2*year), indRandom=2:3)
mj6 <- jointModel(lmeObject=ml6, survObject=ms1, timeVar='year', method='weibull-PH-aGH',
                  parameterization='both', derivForm=dFormQ)
summary(mj6)
anova(mj5, mj6)

?survfitJM
?plot.survfitJM

ND <- subset(pbc2, id==15)
survPreds <- vector('list', nrow(ND))
for(i in 1:nrow(ND)){
  set.seed(4)
  survPreds[[i]] <- survfitJM(mj6, newdata=ND[1:i,])}

par(mfrow=c(1,3))
for(i in c(3,6,9)){
  plot(survPreds[[i]], estimator='median', conf.int=TRUE, include.y=TRUE, pch=19, lwd=2, col='blue',
       xlab='', ylab='', ylab2='', main=paste(round(survPreds[[i]]$last.time, 1), 'years'))}
mtext('Time, years', side=1, line=-2, outer=TRUE)
mtext('log serum bilirubin', side=2, line=-1, outer=TRUE)
mtext('Survival probability', side=4, line=-1, outer=TRUE)

print(sessionInfo(), locale = FALSE)
